```
*******************************************************************************
** INTEL CONFIDENTIAL
**
** Copyright (c) 2020-2023 Intel Corporation All Rights Reserved
**
** This software and the related documents are Intel copyrighted materials,
** and your use of them is governed by the express license under which they
** were provided to you ("License"). Unless the License provides otherwise,
** you may not use, modify, copy, publish, distribute, disclose or transmit
** this software or the related documents without Intel's prior written
** permission.
**
** The source code contained or described herein and all documents related to
** the source code ("Material") are owned by Intel Corporation or its suppliers
** or licensors.
**
** This software and the related documents are provided as is,
** with no express or implied warranties, other than those that are expressly
** stated in the License.
**
*******************************************************************************
```

# IPF Extensible Framework (IPF-EF) SDK

The Innovation Platform Framework enables applications to easily access platform information using a node-based API.

### Hardware Dependencies
Requires Intel's Alder Lake and newer platforms.

### Software Dependencies
1. Microsoft Windows 10 (Version 20H2 and higher) operating system.
2. .NET Framework 4.7.2
3. CMake build system is used for compilation and installation. The required minimum version is 3.15, which may be downloaded from https://cmake.org/download/
4. For development, Visual Studio 2019 and Visual C++ 2019 are required. Install the C# compiler VS2019 for C# client applications.
5. Install the Windows Driver Kit (WDK) component of Visual Studio. It is used for self-signing of DLLs.
6. Run [VC_Redist.x64.exe](https://docs.microsoft.com/en-us/cpp/windows/latest-supported-vc-redist?view=msvc-170) from Microsoft to ensure Visual Studio Runtime is Version 2019 or later

## Getting Started
Steps to install framework and execute of samples in the SDK.

### Install Framework
From the ipf-[version]-windows-setup package, execute `setup.cmd` as Administrator. Here is an example output:

```
 setup.cmd
***
*** Install Intel(R) Innovation Platform Framework Drivers
***

*** Installing ipf_acpi.inf ...
Microsoft PnP Utility

Adding driver package:  ipf_acpi.inf
Driver package added successfully.
Published Name:         oem178.inf

Total driver packages:  1
Added driver packages:  1


*** Installing ipf_cpu.inf ...
Microsoft PnP Utility

Adding driver package:  ipf_cpu.inf
Driver package added successfully.
Published Name:         oem179.inf

Total driver packages:  1
Added driver packages:  1


*** Installing ipf_ef_ext.inf ...
Microsoft PnP Utility

Adding driver package:  ipf_ef_ext.inf
Driver package added successfully.
Published Name:         oem183.inf
Driver package installed on device: PCI\VEN_8086&DEV_1903&SUBSYS_22AF17AA&REV_0C\3&11583659&0&20

Total driver packages:  1
Added driver packages:  1


*** Installing ipf_ef_sw.inf ...
Microsoft PnP Utility

Adding driver package:  ipf_ef_sw.inf
Driver package added successfully.
Published Name:         oem184.inf
Driver package installed on device: SWD\DRIVERENUM\{002c4697-5b34-4222-b9c9-6277b0869281}#IpfEfExtComponent&4&276195b1&7

Total driver packages:  1
Added driver packages:  1

***
*** Install Complete
***
```

### Uninstall Framework
From ipf-[version]-windows-setup package, execute `setup.cmd uninstall` as Administrator.

### One Time Signing Setup
All Provider DLL's, including Local and Debug versions, must be Digitally Signed.
1. Unzip the SDK package contents, ipf-ef-[version]-sdk
2. From tools folder, right-click "certgen.cmd" and select "Run as Administrator". You will be prompted to create an optional certificate password.
3. It is recommended to remove and store ipftest.pvk in a safe place.

Below is an example output, refer to provider guide for further details on signing and certificate distribution.
``` 
Succeeded
Root "Trusted Root Certification Authorities"
CertUtil: -delstore command completed successfully.
Root "Trusted Root Certification Authorities"
Signature matches Public Key
Certificate "IPF Test Certificate" added to store.
CertUtil: -addstore command completed successfully.
***
*** Test Certificate Successfully Generated and Installed
***

```

### Build Sample Providers and Run Client Applications
1. Unzip SDK package contents, ipf-ef-[version]-sdk.zip
2. Launch a Developer Command Prompt for Visual Studio 2019. Run `cmake -S . -B build` from unzipped folder.
3. Run`cmake --build build --config Debug` OR `cmake --build build --config Release`
4. Execute the sample applications
  - `cd build\Debug` OR `cd build\Release`
  - `SampleClientAppCpp.exe` to run the C++ sample application
  - `SampleClientAppC.exe` to run the C sample application
  - `SampleClientAppCsharp.exe` to run the C# sample application

# Getting Started with the Provider Development

There are a few sample providers included inside the SDK package at `samples\provider`. Provider developers need to create three files to integrate with IPF.
* [ProviderName]Api.cpp – The framework requires provider developers to define a manifest and declare a factory entry to create provider.
* [ProviderName]Provider.h - [ProviderName]Provider class is inherited from ProviderDelegate. 
* [ProviderName]Provider.cpp – Provider developers implement exposed methods to integrate IPF framework with their own software stacks.

Providers can be installed with INF or tool based methods. Framework also supports local providers for easy development. Refer to Provider Guide for details on provider installation methods and APIs. 

# Getting Started with Client Application

Framework supports C, C++ and C# client applications. Sample client applications for these languages are included in `samples\client`. The sample clients applications load the sample camera and battery provider as local providers. This means that those shared libraries will be required in the folder where the sample application is run.

Refer to Client Guide for details on Client facing APIs. 

## Troubleshooting Guide
1. Error MSB8040: Spectre-mitigated libraries are required for this project 

    Install latest "Libs for Spectre" from the Visual Studio installer (Individual components tab) for both visual stuido and build tools.

2. Got IpfException when accessing the namespace: Error Locating Framework Directory

   Does software components in device manager has `Intel(R) Innovation Platform Framework Extensible Framework`?

   If it doesn't then re-install framework:
   1. From ipf-[version]-windows-setup package, execute `setup.cmd uninstall` as Admin.
   2. From ipf-[version]-windows-setup package, execute `setup.cmd` as Admin.

3. Exception when accessing the namespace: Failed to load library: ClientApiProxy.dll : 14 : Incompatible Version
   
   This indicates mismatch between framework and SDK versions. Try installing framework and SDK from same IPF release to fix this.

4. Error(42): Invalid Digital Signature

   Try generating new security certs and signing DLLs again. 

   1. Unzip contents of SDK package, ipf-ef-[version]-sdk
   2. From tools folder, right-click "certgen.cmd" and select "Run as Administrator".
   3. From ipf-ef-[version]-sdk folder, execute`cmake --build build --config Debug` OR `cmake --build build --config Release`

5. Failed to run certgen.cmd

    Run certgen.cmd from Visual Studio 2019 powershell.
    1. Open Developer Powershell or Developer Command Prompt for Visual Studio 2019 as Administrator
    2. Navigate to ipf-ef-[version]-sdk/tools
    3. Run certgen.cmd

6. System Error of either VCRuntime140.dll not found or MSCVP140.dll was not found

   This indicates the VC_redist is not installed. Please run [VC_Redist.x64.exe](https://docs.microsoft.com/en-us/cpp/windows/latest-supported-vc-redist?view=msvc-170) install VC2019 or later. 

